/**
This software and ancillary information (herein called "SOFTWARE")
called Supermon is made available under the terms described
here.  The SOFTWARE has been approved for release with associated
LA-CC Number LA-CC 99-51.

Unless otherwise indicated, this SOFTWARE has been authored by an
employee or employees of the University of California, operator of the
Los Alamos National Laboratory under Contract No.  W-7405-ENG-36 with
the U.S. Department of Energy.  The U.S. Government has rights to use,
reproduce, and distribute this SOFTWARE, and to allow others to do so.
The public may copy, distribute, prepare derivative works and publicly
display this SOFTWARE without charge, provided that this Notice and
any statement of authorship are reproduced on all copies.  Neither the
Government nor the University makes any warranty, express or implied,
or assumes any liability or responsibility for the use of this
SOFTWARE.

If SOFTWARE is modified to produce derivative works, such modified
SOFTWARE should be clearly marked, so as not to confuse it with the
version available from LANL.
**/
/** NOTICE: This software is licensed under the GNU Public License, which
    is included as LICENSE_GPL in this source distribution. **/
/** NOTE: This library is part of the supermon project, hence the name
          supermon above. **/
#include <stdio.h>
#include <fcntl.h>
#include <string.h>
//++pac
#ifdef _PLAN9
#include "../src/sexp.h"
#else
#include "sexp.h"
#endif

//++pac
#ifdef _PLAN9
#include "../src/sexp_ops.h"
#else
#include "sexp_ops.h"
#endif


int main(int argc, char **argv) {
  sexp_t *a, *b, *c, *d;
  char sx_text[2048], pstr[1024];
  
  sprintf(sx_text,"((a1 a2) b c (a1 b1))");
  a = parse_sexp(sx_text,strlen(sx_text));
  sprintf(sx_text,"(d e f)");
  b = parse_sexp(sx_text,strlen(sx_text));
  c = cons_sexp(a,b);
  print_sexp(pstr,1024,a);
  printf("a=[%s]\n",pstr);
  print_sexp(pstr,1024,b);
  printf("b=[%s]\n",pstr);
  print_sexp(pstr,1024,c);
  printf("cons(a,b)=[%s]\n",pstr);
  
  d = bfs_find_sexp("a1",a);
  print_sexp(pstr,1024,d);
  printf("bfs(\"a1\",a)=%s\n",pstr);
  destroy_sexp(b);
  destroy_sexp(c);
  
  b = car_sexp(a);
  print_sexp(pstr,1024,b);
  printf("car(a)=[%s]\n",pstr);
  
  c = cdr_sexp(a);
  print_sexp(pstr,1024,c);
  printf("cdr(a)=[%s]\n",pstr);
  
  d = cons_sexp(b,c);
  print_sexp(pstr,1024,d);
  printf("cons(car(a),cdr(a))=[%s]\n",pstr);
  
  destroy_sexp(c);
  destroy_sexp(d);
  destroy_sexp(b);
  destroy_sexp(a);
  
  exit(1);
}
