/**
This software and ancillary information (herein called "SOFTWARE")
called Supermon is made available under the terms described
here.  The SOFTWARE has been approved for release with associated
LA-CC Number LA-CC 99-51.

Unless otherwise indicated, this SOFTWARE has been authored by an
employee or employees of the University of California, operator of the
Los Alamos National Laboratory under Contract No.  W-7405-ENG-36 with
the U.S. Department of Energy.  The U.S. Government has rights to use,
reproduce, and distribute this SOFTWARE, and to allow others to do so.
The public may copy, distribute, prepare derivative works and publicly
display this SOFTWARE without charge, provided that this Notice and
any statement of authorship are reproduced on all copies.  Neither the
Government nor the University makes any warranty, express or implied,
or assumes any liability or responsibility for the use of this
SOFTWARE.

If SOFTWARE is modified to produce derivative works, such modified
SOFTWARE should be clearly marked, so as not to confuse it with the
version available from LANL.
**/
/** NOTE: This library is part of the supermon project, hence the name
          supermon above. **/
/**
 * very rough python binding for sexpr library.  used for testing.
 **/
#include "Python.h"
#include <stdio.h>
#include "sexpr.h"

/**
 * last expression parsed and last one found
 */
sexpr_t *last_sexpr;
sexpr_t *last_find;

PyObject *destroy_wrap(PyObject *self, PyObject *args) {
  if (last_sexpr != NULL)
    destroy_sexp(last_sexpr);

  last_sexpr = NULL;
  last_find = NULL;

  Py_INCREF(Py_None);
  return Py_None;
}

PyObject *find_wrap(PyObject *self, PyObject *args) {
  char *str;

  if (!PyArg_ParseTuple(args,"s",&str)) {
        return NULL;
  } 
 
  last_find = find_sexp(str,last_sexpr);

  Py_INCREF(Py_None);
  return Py_None;
}

PyObject *parse_wrap(PyObject *self, PyObject *args) {
  char *str;
  int len;

  if (!PyArg_ParseTuple(args,"s#",&str,&len)) {
        return NULL;
  } 
 
  last_sexpr = parse_sexp(str,len);

  Py_INCREF(Py_None);
  return Py_None;
}

PyObject *print2_wrap(PyObject *self, PyObject *args) {
  char str[4096];

  if (last_find == NULL) {
    Py_INCREF(Py_None);
    return Py_None;
  } else {
    print_sexp(str,4096,last_find);
    return Py_BuildValue("s",str);
  }
}

PyObject *print_wrap(PyObject *self, PyObject *args) {
  char str[4096];

  if (last_sexpr == NULL) {
    Py_INCREF(Py_None);
    return Py_None;
  } else {
    print_sexp(str,4096,last_sexpr);
    fprintf(stderr,"STRING==[%s]\n",str);
    return Py_BuildValue("s",str);
  }
}

static PyMethodDef sexprmethods[] = {
	{"parser", parse_wrap, METH_VARARGS},
  	{"printer", print_wrap, METH_VARARGS},
  	{"printer2", print2_wrap, METH_VARARGS},
  	{"destroy", destroy_wrap, METH_VARARGS},
  	{"finder", find_wrap, METH_VARARGS},
	{NULL,NULL}
};

void initsexpr() {
  Py_InitModule("sexpr", sexprmethods);
  last_sexpr = NULL;
  last_find = NULL;
}
